"""Skew-T/Log-P plotting, meteorology and interfacing with CM1

This package provides routines for plotting Skew-T/Log-P diagrams
and working with CM1 output.  For contributions, please submit
pull requests to https://github.com/cwebster2/pymeteo. Please report
bugs using the github issue tracker at https://github.com/cwebster2/pymeteo/issues.

"""
__version__="1.0.1"
